// Posedge-One Comprehensive SRAM test
// 
module sram_test
   (
    input wire CLOCK, RESET_N,
    input wire [7:0] DIP,
    input wire [3:0] PB,
    output wire [3:0] AN,
    output wire [7:0] LED, SEG,
    output wire [18:0] SRAM_ADDR,
    output wire SRAM_WE_N, SRAM_OE_N,
    inout wire [7:0] SRAM_DATA,
    output wire SRAM_CE_N
   );

   // symbolic state declaration
   localparam [2:0]
      test_init = 3'b000,
      rd_clk1   = 3'b001,
      rd_clk2   = 3'b010,
      rd_clk3   = 3'b011,
      wr_err    = 3'b100,
      wr_clk1   = 3'b101,
      wr_clk2   = 3'b110,
      wr_clk3   = 3'b111;

   // signal declaration
	(*keep="TRUE"*)
   reg [2:0] state_reg, state_next;
   reg [18:0] addr;
   wire [7:0] data_s2f;
   reg [7:0] data_f2s;
   reg mem, rw;
   reg [18:0] c_next, c_reg;
   reg [7:0] inj_next, inj_reg;
   reg [15:0] err_next, err_reg;
	wire reset, clk;
	wire [3:0] btn;

   // body
   //===============================================
   // component instantiation
   //===============================================
   // instantiation
   sram_ctrl ctrl_unit
      (.clk(clk), .reset(reset), .mem(mem), .rw(rw),
       .addr(addr), .data_f2s(data_f2s), .ready(),
       .data_s2f_r(), .data_s2f_ur(data_s2f), .ad(SRAM_ADDR),
       .we_n(SRAM_WE_N), .oe_n(SRAM_OE_N), .dio_a(SRAM_DATA),
       .ce_a_n(SRAM_CE_N));

   
   disp_hex_mux disp_unit
      (.clk(clk), .reset(1'b0), .dp_in(4'b1111),
       .hex3(err_reg[15:12]), .hex2(err_reg[11:8]),
       .hex1(err_reg[7:4]), .hex0(err_reg[3:0]),
       .an(AN), .sseg(SEG));
	
	 pll pll_i
   (// Clock in ports
    .CLK_IN1(CLOCK),      // IN
    // Clock out ports
    .CLK_OUT1(clk)); 

   //===============================================
   //   FSMD
   //===============================================
   // FSMD state & data registers
   always @(posedge clk, posedge reset)
      if (reset)
         begin
            state_reg <= test_init;
            c_reg <= 0;
            inj_reg <= 0;
            err_reg <= 0;
         end
      else
         begin
            state_reg <= state_next;
            c_reg <= c_next;
            inj_reg <= inj_next;
            err_reg <= err_next;
         end
   // FSMD next-state logic
   always @*
   begin
      c_next = c_reg;
      inj_next = inj_reg;
      err_next = err_reg;
      addr = 0;
      rw = 1'b1;
      mem = 1'b0;
      data_f2s = 0;
      case (state_reg)
         test_init:
            if (btn[0])
               begin
                  state_next = rd_clk1;
                  c_next = 0;
                  err_next = 0;
               end
            else if (btn[1])
               begin
                  state_next = wr_clk1;
                  c_next = 0;
                  inj_next = 0;
               end
            else if (btn[2])
               begin
                  state_next = wr_err;
                  inj_next = inj_reg + 1;
               end
            else
               state_next = test_init;
         wr_err: // write 1 error; done in next 2 clocks
            begin
               state_next = test_init;
               mem = 1'b1;
               rw = 1'b0;
               addr = {11'b0, DIP};
               data_f2s = 8'hff;
            end
         wr_clk1: // in idle state of sram_ctrl
            begin
               state_next = wr_clk2;
               mem = 1'b1;
               rw = 1'b0;
               addr = c_reg;
               data_f2s = ~c_reg[7:0];
            end
         wr_clk2: // in wr1 state of sram_ctrl
            state_next = wr_clk3;
         wr_clk3: // in wr2 state of sram_ctrl
            begin
               c_next = c_reg + 1;
               if (c_next==0)
                  state_next = test_init;
               else
                  state_next = wr_clk1;
            end
         rd_clk1: // in idle state of sram_ctrl
            begin
               state_next = rd_clk2;
               mem = 1'b1;
               rw = 1'b1;
               addr = c_reg;
            end
         rd_clk2: // in rd1 state of sram_ctrl
            state_next = rd_clk3;
         rd_clk3: // in rd2 state of sram_ctrl
            begin
               // compare readout; must use unregistered output
               if (~c_reg[7:0] != data_s2f)
                  err_next = err_reg + 1;
               c_next = c_reg + 1;
               if (c_next==0)
                  state_next = test_init;
               else
                  state_next = rd_clk1;
            end
     endcase
   end
   // output
   assign LED = inj_reg;
	assign reset = ~RESET_N;
	assign btn = ~PB;

endmodule

